#!D:\bld\gdal-split_1632408703732\_h_env\python.exe
# EASY-INSTALL-SCRIPT: 'GDAL==3.2.2','gdal_proximity.py'
__requires__ = 'GDAL==3.2.2'
__import__('pkg_resources').run_script('GDAL==3.2.2', 'gdal_proximity.py')
